"""
Module contains functions to use for work with settings
"""

import configparser
import os
from argparse import Namespace
from typing import Dict, Optional, Tuple, Union

from configs_core.functions import getConfiguratorAddressFromFile, parseConfiguratorAddress, pullConfigFromConfigurator

from .all_settings import SETTINGS


def getTagsFromCliParser(cmdOptions: Namespace, serviceName: str) -> Dict[str, str]:
    """
    Get dict with pairs of setting names and values for settings with specified tags
    Args:
        cmdOptions: namespace with cmd options
        serviceName: service name

    Returns:
        dict with setting names and tags
    """
    serviceSettingList = [
        settingName
        for settingName, settingValue in SETTINGS.items()
        if serviceName in settingValue.services and settingName in cmdOptions.__dict__
    ]
    fullDict = dict(zip(serviceSettingList, [cmdOptions.__dict__[settingName] for settingName in serviceSettingList]))
    result = {key: value for key, value in fullDict.items() if value is not None}
    return result


def getConfiguratorAddressFromCmd(cmdOptions: Namespace) -> Tuple[str, int]:
    """
    Get configurator address from command line arguments.

    Args:
        cmdOptions: command line options

    Returns:
        tuple<origin, apiVersion>
    """
    return parseConfiguratorAddress(cmdOptions.luna_config)


def getConfigFromFile(configPath: Optional[str] = None) -> configparser.ConfigParser:
    """
    Get config from file (default if not specified)
    Args:
        configPath: path to config file
    Returns:
        config from file
    """
    configPath = (
        configPath
        if configPath is not None
        else os.path.join(os.path.abspath(os.path.dirname(__file__)), os.path.pardir, os.path.pardir, "config.conf")
    )
    configParser = configparser.ConfigParser()
    if not os.path.isfile(configPath):
        raise RuntimeError(f"Configuration file {configPath} not found")
    configParser.read(configPath)
    return configParser


def getConfigWithSource(cmdOptions: Namespace) -> Tuple[Optional[configparser.ConfigParser], bool]:
    """
    Get config source and config location flag
    Args:
        cmdOptions: cmdOptions

    Returns:
        Tuple with:
            - parsed config if config from file else None
            - True if config from file otherwise False
    """
    if getattr(cmdOptions, "luna_config", None) is not None:
        return None, False
    configFromFile = getConfigFromFile(cmdOptions.config)
    if configFromFile.has_section("LUNA_CONFIGURATOR") and int(
        configFromFile["LUNA_CONFIGURATOR"].get("USE_CONFIGURATOR", "0")
    ):
        return configFromFile, False
    return configFromFile, True


def getConfiguratorAddressFromConfig(
    parsedConfig: configparser.ConfigParser,
) -> Union[Tuple[str, int], Tuple[None, None]]:
    """
    Get configurator origin and api from config file
    Args:
        parsedConfig: parsed config from file

    Returns:
        configurator origin, api or tuple with none if configurator is not using
    """
    if parsedConfig.has_section("LUNA_CONFIGURATOR"):
        configuratorSection = dict(parsedConfig["LUNA_CONFIGURATOR"])
    else:
        configuratorSection = {}
    return getConfiguratorAddressFromFile(configuratorSection)


def getConfiguratorAddress(
    cmdOptions: Namespace, parsedConfig: configparser.ConfigParser
) -> Union[Tuple[str, int], Tuple[None, None]]:
    """
    Get configurator origin and api from cmd or config file
    Args:
        cmdOptions: cmd options
        parsedConfig: parsed config from file

    Returns:
        configurator origin, api or tuple with none if configurator is not using
    """
    if getattr(cmdOptions, "luna_config", None) is not None:
        configuratorOrigin, configuratorApiVersion = getConfiguratorAddressFromCmd(cmdOptions)
    else:
        configuratorOrigin, configuratorApiVersion = getConfiguratorAddressFromConfig(parsedConfig)
    return configuratorOrigin, configuratorApiVersion


def getSettings(
    serviceName: str,
    cmdOptions: Namespace,
    isConfigFromFile: bool,
    parsedConfig: Optional[configparser.ConfigParser] = None,
) -> Union[configparser.ConfigParser, dict]:
    """
    Get config for required service
    Args:
        serviceName: required service name
        cmdOptions: cmd options
        isConfigFromFile: if true read config from file else get config from configurator
        parsedConfig: parsed config from file

    Returns:
        config for required service
    """
    if isConfigFromFile:
        return getConfigFromFile(cmdOptions.config)

    configuratorOrigin, configuratorApiVersion = getConfiguratorAddress(cmdOptions, parsedConfig)
    return pullConfigFromConfigurator(
        serviceName, configuratorOrigin, configuratorApiVersion, getTagsFromCliParser(cmdOptions, serviceName)
    )


def getAllSettingNames() -> set:
    """
    Get all possible settings' names
    Returns:
        set of setting names
    """
    res = set()
    for settingName, settingDict in SETTINGS.items():
        if "oneOf" in settingDict.schema:
            for oneOfSchema in settingDict.schema["oneOf"]:
                for propertyName in oneOfSchema["properties"]:
                    res.add(propertyName.upper())
        elif "properties" in settingDict.schema:
            for propertyName in settingDict.schema["properties"]:
                res.add(propertyName.upper())
        else:
            res.add(settingName.upper())
    return res
