"""
Test config loader functionality.
"""

import os
from collections import namedtuple
from operator import attrgetter
from tempfile import gettempdir
from typing import Any, Dict, Iterable, List, Optional, Set, Union
from unittest import TestCase
from unittest.mock import patch

import fastjsonschema
from fastjsonschema import validate

from tests.data.luna_test_configs import (
    DEFAULT_VALUES,
    LARGE_NESTED_SETTING_SCHEMA,
    TEST_VALUES,
    SettingsTest,
    fileConf,
    impossibleOneOfFileConf,
    impossibleOneOfPullerConf,
    minimalFileConf,
    minimalPullerConf,
    pullerConf,
    secondOneOfFileConf,
    secondOneOfPullerConf,
)


def genPaths(obj: Union[dict, list], path: Optional[List[str]] = None) -> Iterable[List[Union[str, int]]]:
    """
    Generate Use with `getPath`.

    Args:
        obj: any object
        path:

    Yields:
        existing paths in the object
    """
    if path is None:
        path = []
    if isinstance(obj, (dict, list)):
        KVGen = obj.items() if isinstance(obj, dict) else enumerate(obj)
        for key, value in KVGen:
            yield from genPaths(value, path + [key])
    elif isinstance(obj, (int, float, str, type(None))):
        yield path
    else:
        raise ValueError(f"Unsupported type: {type(obj)}")


def getPath(obj: Union[dict, list], path: List[Union[str, int]]) -> Any:
    """
    Get a nested object by path, generated by `genPaths`.

    Args:
        obj: root object
        path: a  path in root object

    Returns:
        nested object
    """
    nested = obj
    for key in path:
        nested = nested[key]
    return nested


class FunctionalityTest(TestCase):
    maxDiff = None  # unlimited unittest validation traces
    confFileLocation = os.path.join(gettempdir(), "test_config.conf")  # temporary config file path

    @classmethod
    def tearDownClass(cls) -> None:
        super().tearDownClass()
        if os.path.exists(cls.confFileLocation):
            os.remove(cls.confFileLocation)

    @staticmethod
    def getPullerConfig(pullerJson: Dict[str, Any]) -> SettingsTest:
        """
        Get `SettingsTest` instance from puller json.

        Args:
            pullerJson: puller response json

        Returns:
            generated SettingsTest
        """
        pullerResponse = namedtuple("Response", "success json")(True, pullerJson)

        async def patchPuller(*a, **kw):
            return pullerResponse

        with patch("luna3.configurator.configurator.ConfiguratorApi.pullConfig", patchPuller):
            fromPuller = SettingsTest()
            fromPuller.load(configuratorAddress="http://127.0.0.1:5070/1")
        return fromPuller

    def getFileConfig(self, fileData: str) -> SettingsTest:
        """
        Get `SettingsTest` instance from config file data.

        Args:
            fileData: config file data

        Returns:
            generated SettingsTest
        """
        with open(self.confFileLocation, "w") as f:
            f.write(fileData)
        fromFile = SettingsTest()
        fromFile.load(configFile=self.confFileLocation)
        return fromFile

    def assertConfig(self, config: SettingsTest, setOfNestedValues: Union[Set[Any], None], ignoreAttrs: List[str] = ()):
        """
        Assert config correctness:
            1. assert `largeNestedSettings` attribute existence
            2. validate largeNestedSettings using LARGE_NESTED_SETTING_SCHEMA
            3. assert deepest-level values to be in `setOfNestedValues`

        Args:
            config: config to assert
            setOfNestedValues: full set of deepest-level values in the config object
            ignoreAttrs: setting instance attributes to ignore
        """
        self.assertTrue(hasattr(config, "largeNestedSettings"))
        validate(LARGE_NESTED_SETTING_SCHEMA, config._inputLargeNestedSettings)

        if setOfNestedValues is not None:
            realNestedValues = set()
            for attrName in config.largeNestedSettings.__slots__:
                if attrName in ignoreAttrs:
                    continue
                attrPath = f"largeNestedSettings.{attrName}"
                attr = attrgetter(attrPath)(config)
                for path in genPaths(attr):
                    with self.subTest(path=[attrPath] + path):
                        self.assertIn(getPath(attr, path), setOfNestedValues)
                    realNestedValues.add(getPath(attr, path))
            self.assertSetEqual(setOfNestedValues, realNestedValues)

    def assertConfigsEqual(
        self,
        firstConfig: SettingsTest,
        secondConfig: SettingsTest,
        setOfNestedValues: Union[Set[Any], None],
        ignoreAttrs: List[str] = (),
    ):
        """
        Assert configs equality:
            1. assert first config
            2. assert second config
            3. assert data equality in configs (using first config's skeleton)

        Args:
            firstConfig: first config
            secondConfig: second config
            setOfNestedValues: full set of deepest-level values in the config object
            ignoreAttrs: setting instance attributes to ignore
        """
        self.assertConfig(firstConfig, setOfNestedValues=setOfNestedValues, ignoreAttrs=ignoreAttrs)
        self.assertConfig(secondConfig, setOfNestedValues=setOfNestedValues, ignoreAttrs=ignoreAttrs)

        self.assertEqual(firstConfig, secondConfig)

    def test_conf(self):
        """
        Test the `.conf` file parsing.
        """
        fileConfig = self.getFileConfig(fileConf)
        self.assertConfig(fileConfig, setOfNestedValues=set(TEST_VALUES.values()), ignoreAttrs=["importThis"])

    def test_conf_defaults(self):
        """
        Test the empty `.conf` file parsing.
        """
        fileConfig = self.getFileConfig(minimalFileConf)
        self.assertConfig(fileConfig, setOfNestedValues=set(DEFAULT_VALUES.values()), ignoreAttrs=["importThis"])

    def test_puller(self):
        """
        Test the puller response parsing.
        """
        pullerConfig = self.getPullerConfig(pullerConf)
        self.assertConfig(pullerConfig, setOfNestedValues=set(TEST_VALUES.values()), ignoreAttrs=["importThis"])

    def test_puller_defaults(self):
        """
        Test the empty puller response parsing.
        """
        pullerConfig = self.getPullerConfig(minimalPullerConf)
        self.assertConfig(pullerConfig, setOfNestedValues=set(DEFAULT_VALUES.values()), ignoreAttrs=["importThis"])

    def test_puller_and_config_equality(self):
        """
        Compare the `.conf` file parsing and the puller response parsing results.
        """
        fileConfig = self.getFileConfig(fileConf)
        pullerConfig = self.getPullerConfig(pullerConf)
        self.assertConfigsEqual(
            fileConfig, pullerConfig, setOfNestedValues=set(TEST_VALUES.values()), ignoreAttrs=["importThis"]
        )

    def test_puller_and_config_defaults_equality(self):
        """
        Compare the empty `.conf` file parsing and the empty puller response parsing results.
        """
        fileConfig = self.getFileConfig(minimalFileConf)
        pullerConfig = self.getPullerConfig(minimalPullerConf)
        self.assertConfigsEqual(
            fileConfig, pullerConfig, setOfNestedValues=set(DEFAULT_VALUES.values()), ignoreAttrs=["importThis"]
        )

    def test_second_one_of(self):
        """
        Load second oneOf config.
        """
        fileConfig = self.getFileConfig(secondOneOfFileConf)
        pullerConfig = self.getPullerConfig(secondOneOfPullerConf)
        setOfNestedValues = {"this"}  # only "import":"this" is there
        self.assertConfigsEqual(
            fileConfig, pullerConfig, setOfNestedValues=setOfNestedValues, ignoreAttrs=["someRootField", "someData"]
        )
        self.assertIsNone(fileConfig.largeNestedSettings.someData)
        self.assertIsNone(fileConfig.largeNestedSettings.someRootField)
        self.assertIsNone(pullerConfig.largeNestedSettings.someData)
        self.assertIsNone(pullerConfig.largeNestedSettings.someRootField)

    def test_impossible_one_of(self):
        """
        Load second oneOf config.
        """
        for readerName, reader in [
            ("file", lambda: self.getFileConfig(impossibleOneOfFileConf)),
            ("puller", lambda: self.getPullerConfig(impossibleOneOfPullerConf)),
        ]:
            with self.subTest(readerName=readerName):
                with self.assertRaises(fastjsonschema.exceptions.JsonSchemaException) as e:
                    reader()
                self.assertEqual(
                    "No suitable oneOf schema for setting {'some_root_field': 'test', 'import': 'this'}",
                    e.exception.args[0],
                )
