import inspect
import os
import re
import sys
from unittest import TestCase

from fastjsonschema import validate

from .. import helpers

sys.path.append(os.path.dirname(os.path.dirname(__file__)))


class TestConfigsCorrectness(TestCase):
    """
    Test settings' correctness.
    """

    def test_service_registration(self):
        """
        Check if the service name is added to all_settings in configs submodule.
        """
        from ..configs.settings.all_settings import PYTHON_SERVICES as all_services

        service_name = helpers.get_service_name()
        self.assertIn(service_name, all_services, msg="add service to all_settings in configs submodule")

    def test_service_config_class(self):
        """
        Check the availability of ServiceSettings class in service_config.py.
        """
        try:
            from ...service_config import ServiceSettings

            ServiceSettings()
        except FileNotFoundError:
            self.fail(msg="add service_configs.py to service_app_folder/configs")
        except AttributeError:
            self.fail(msg="add ServiceSettings class to service_config.py")
        except ImportError:
            self.fail(msg="remove relative imports in service_config.py")

    def test_settings_correctness(self):
        """
        Check each `setting` has the same name as the key in the `SETTINGS` dict.
        """
        from ..configs.settings.all_settings import SETTINGS as settings

        for setting_name, setting in settings.items():
            with self.subTest(setting=setting_name):
                self.assertTrue(setting_name.isupper(), "set setting name upper")
                # I need private attribute, so... pylint: disable-msg=W0212
                self.assertEqual(setting_name, setting._settingName, "set setting name the same")

    def test_services_completeness(self):
        """
        Check `services` field in each `setting` (i.e. DATABASE_NUMBER) to be in the corresponding `service adapter`
        (i.e. `SettingsFaces`).
        """
        from configs_core.base_classes import BaseSettingClass
        from configs_core.base_service_setting import BaseServiceSettingsClass as CoreBaseServiceSettingsClass

        # from configs.settings.all_settings import SETTINGS as settings
        from ..configs.settings.all_settings import SETTINGS as settings

        def get_constants(cls, service):
            """
            Get all `str` constants from the class __init__ method.

            Args:
                cls: class to scan
                service: service name

            Returns:
                constants
            """
            service_prefix = service.upper().replace("-", "_")
            strings = {const.upper() for const in cls.__init__.__code__.co_consts if isinstance(const, str)}
            if hasattr(cls, "applySettings"):
                strings.update(
                    {const.upper() for const in cls.applySettings.__code__.co_consts if isinstance(const, str)}
                )
            if hasattr(cls, "applyEstimatorsSettings"):
                strings.update(
                    {
                        const.upper()
                        for const in cls.applyEstimatorsSettings.__code__.co_consts
                        if isinstance(const, str)
                    }
                )
            good_strings = {string for string in strings if re.match(r"^[_a-zA-Z][a-zA-Z0-9_\-]*$", string) is not None}
            good_strings = {
                string if not string.startswith("_") else f"{service_prefix}{string}" for string in good_strings
            }
            return good_strings

        def is_needed_class(cls):
            """
            Whether we need a class.

            Args:
                cls: class

            Returns:
                need
            """
            if not inspect.isclass(cls):
                return False
            return any(
                (
                    issubclass(cls, BaseSettingClass),
                    issubclass(cls, CoreBaseServiceSettingsClass),
                )
            )

        service_name = helpers.get_service_name()
        from ... import service_config

        for setting_name, setting in settings.items():
            if setting_name == "LUNA_CONFIGURATOR":
                continue
            if service_name in setting.services:
                with self.subTest(setting=setting_name, service=service_name):
                    constants = set()
                    for _, module_object_class in inspect.getmembers(service_config, is_needed_class):
                        constants |= get_constants(module_object_class, service_name)
                    self.assertTrue(
                        setting_name in constants,
                        f"Seems like the setting '{setting_name}' " f"is not used in the '{service_name}' service",
                    )

    def test_default_values(self):
        """
        Check default values.
        """
        from ..configs.settings.all_settings import SETTINGS as settings

        for setting in settings.values():
            with self.subTest(setting_name=setting._settingName):
                validate(setting.schema, setting.default)
